"""IBM WebSphere Application Server Patterns

  Copyright IBM Corp. 2015
  All rights reserved.
  US Government Users Restricted Rights -
  Use, duplication or disclosure restricted
  by GSA ADP Schedule Contract with IBM Corp.

  Licensed Materials-Property of IBM
"""
import traceback
from com.ibm.websphere.management.cmdframework import CommandNotFoundException
from com.ibm.ws.sm.workspace import WorkSpaceException
from com.ibm.ws.scripting import ScriptingException
from com.ibm.websphere.management.exception import ConfigServiceException
from com.ibm.websphere.management.exception import ConnectorException
from java.security import PrivilegedActionException

def findCell ( cellName ):
    global AdminApp, AdminConfig

    oCellToUse = findConfigTarget(cellName, "Cell" )

    return oCellToUse
#endDef

def findCellName():
    global AdminAdminConfig

    element = AdminConfig.list("Cell")
    name = AdminConfig.showAttribute(element,"name")
    print " ** findCellName returning " + name

    return name
#endDef

#--------------------------------------------------------------------
# Procedure:    findNode
# Description:  Determines whether a node exists
#
# Parameters:   nodeName
#
# Returns:  nodeName = success;
#       0 = node does not exist
#--------------------------------------------------------------------
def findNode ( nodeName ):
    global AdminApp, AdminConfig

    oNodeToUse = findConfigTarget(nodeName,"Node")
    return oNodeToUse
#endDef

def findName ( type ):
    global AdminApp, AdminConfig

    elements = AdminConfig.list(type)
    if (elements == " "):
        return " "
    #endIf
    elementList = elements.split(lineSeparator)
    for element in elementList:
        element=element.rstrip()
        if (len(element) > 0 ):
            name=AdminConfig.showAttribute(element,"name")
            return element
        #endIf
    #endFor
    return " "    
#endDef

#--------------------------------------------------------------------
# Procedure:    findServer
# Description:  Determine whether server exists
#
# Parameters:   serverName
#
# Returns:  serverName=success;
#       0 =server does not exist
#--------------------------------------------------------------------
def findServer ( serverName ):
    global AdminApp, AdminConfig

    oServerToUse = findConfigTarget(serverName,"Server")
    return oServerToUse
#endDef

def findCluster():

    global AdminConfig
    
    element = AdminConfig.list("ServerCluster")
    name = AdminConfig.showAttribute(element,"name")

    return name
#enddef    
    

#--------------------------------------------------------------------
# Procedure:    findConfigTarget
# Description:  Determine if there is a config element for given
#           name and type
#
# Parameters:   nameSearch  
#       type        (Cell, Node, JDBCProvider)
#
# Returns:  targetName = success
#       0  = does not exist
#--------------------------------------------------------------------
def findConfigTarget ( nameSearch, type ):
    global AdminApp, AdminConfig

    elements = AdminConfig.list(type)
    if (elements == " "):
        return 0
    #endIf
    elementList = elements.split(lineSeparator)
    for element in elementList:
        element=element.rstrip()
        if (len(element) > 0 ):
            name=AdminConfig.showAttribute(element,"name")

            if (nameSearch == name):
                return element
            #endIf
        #endIf
    #endFor

    return 0
#endDef

#******************************************************************************
# Procedure:    createCluster
# Description:  Create new cluster
# Author:       Gale Botwick - gbotwick@us.ibm.com
#
# History:  
#       
#******************************************************************************
def createCluster ( clusterName, prefLocal):

    #---------------------------------------------------------------------------------
    # Create server cluster if it does not exist
    #---------------------------------------------------------------------------------

    global AdminApp, AdminConfig

    print '\n====== Create cluster '+clusterName+', if it does not exist ======'

    # Check if server cluster already exists
    oClusterToUse = findConfigTarget(clusterName, "ServerCluster" )
    if (oClusterToUse != 0):
        print "Cluster already exists, so no cluster will be created"
        AdminConfig.reset()
        return
    #endIf
    
    cellName = findCellName()
    oCellToUse = findCell(cellName)
    if (oCellToUse == 0):
        print "Cell: " + cellName + " does not exist."
        return
    #endIf

    #---------------------------------------------------------------------
    # Construct the attribute list to be used in creating a server cluster
    #---------------------------------------------------------------------
    attrs = [["name", clusterName], ["preferLocal", prefLocal]]

    #---------------------------------------------------------
    # Create the server cluster
    #---------------------------------------------------------
    _excp_ = 1
    while (_excp_ != 0):    
        try:
            _excp_ = 0
            print "Attempting to create the cluster: " + clusterName
            createCluster = AdminConfig.create("ServerCluster", oCellToUse, attrs )
            result = createCluster
            
            AdminConfig.save()

        except CommandNotFoundException:
            _type_, _value_, _tbck_ = sys.exc_info()
            _value_ = str(_value)
            
            print traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])

            AdminConfig.reset()
        
            _excp_ = 1
            
        except ScriptingException:
            _type_, _value_, _tbck_ = sys.exc_info()
            _value_ = str(_value)
            if _value_.find("already exists") > 0:
                print "Cluster already exists.  Nothing to do"
                print "resetting wsadmin to pick up correct values\n"
                AdminConfig.reset()
                _excp_ = 0
                return

            _excp_ = 1
            
        except (Exception, ConfigServiceException), e:
            print "In Cluster Exception block\n"
            print "Exception:", e

            _type_, _value_, _tbck_ = sys.exc_info()
            _value_ = str(_value)
            
            if _value_.find("DocumentAlreadyExistsException") > 0:
                print "Error while trying to save the config changes.  Will reset and try again"
                AdminConfig.reset()
                _excp_ = 0
            else:
                print "In Exception catch \n"              
                print traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
            
                AdminConfig.reset()
            
                _excp_ = 1

        except:         
            print traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
            
            AdminConfig.reset()
            
            print "unknown error exiting"
            
            return
        #endTry
    #endwhile

    print "Cluster "+clusterName+" created successfully"

#endDef

#******************************************************************************
# Procedure:    createClusterMember
# Description:  Create new cluster member with a given cluster name
# Author:       Gale Botwick - gbotwick@us.ibm.com
#
# History:  03/13/06:  Removed serverTemplate as an input
#******************************************************************************
def createClusterMember ( node_prefix, number_servers, clusterName, nodeName, weightValue):

    #--------------------------------------------------------------------------------------
    # Create cluster member if it does not exist
    #--------------------------------------------------------------------------------------
    global AdminApp, AdminConfig

    _excp_ = 1
    while (_excp_ != 0):    
        try:
            _excp_ = 0
            for i in range(0, int(number_servers)):
                clusterMemberName = node_prefix + node_name + "_" + str(i+1)
                # Check if server already exists
                oServerToUse = findServer(clusterMemberName )
                if (oServerToUse != 0):
                    print "Server " + clusterMemberName + " already exists"
                    continue
                #endIf

                print "Attempting to create the server member " + clusterMemberName
                print "Using Weight:        " + str(weightValue)

                server = AdminTask.createClusterMember(["-clusterName", clusterName, "-memberConfig", [[nodeName, clusterMemberName, str(weightValue)]]])
                
                print "Cluster member " + clusterMemberName + " created successfully"
            #endfor
            
            AdminConfig.save()
            
        except (Exception, AdminTask), e:
            print "In create cluster member Exception block\n"
            print "Exception:", e

            _type_, _value_, _tbck_ = sys.exc_info()
            _type_ = str(_type_)
            _value_ = str(_value_)
            
            print traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])

            if _value_.find("DocumentAlreadyExistsException") > 0:
                print "Error while trying to save the config changes.  Will reset and try again"
                AdminConfig.reset()
                _excp_ = 0
            elif e.find("AdminTask") > 0:
                sys.exit(1)
            elif _value_.find("AdminTask") > 0:
                sys.exit(1)
            else:
                AdminConfig.reset()
            
                _excp_ = 1
                
        except (ConfigServiceException, ScriptingException, PrivilegedActionException, ConnectorException), e:

            print "Exception:", e

            _type_, _value_, _tbck_ = sys.exc_info()
            _value_ = str(_value_)
            
            if _value_.find("while updating documents in master repository") > 0:
                print "Error while trying to save the config changes.  Will reset and try again"

            print traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
           
            AdminConfig.reset()
            
            _excp_ = 1
            
        except:
            
            print "In except catch \n"              
            print sys.exc_info()

            print "unknown error exiting"
            return
        
        #endTry
    #endwhile
#endDef

# START main method
def main (cluster_name, number_servers, node_prefix, node_name):
    print("cluster name is: %s" % cluster_name)
    print("number servers is: %s" % number_servers)
    print("node prefix is: %s" % node_prefix)
    print("node name: " + node_name)

    if (findNode(node_name) == 0):
        print "Node with node name of " + node_name + " could not be found.  Exiting."
        sys.exit(2)
        
    try:
        createCluster(cluster_name, "true")
        createClusterMember(node_prefix, number_servers, cluster_name, node_name, 1)
    except Exception, e:
        print "In main Exception block\n"
        print "Exception:", e
        
        print traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
        
        sys.exit(1)
    except:
        print "In main catch \n"              

        print traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
        
        sys.exit(1)
        
    #endtry
 #enddef

if __name__ == "__main__":
    if 1 < len(sys.argv) :
        cluster_name = sys.argv[0]
        print cluster_name
        number_servers = sys.argv[1]
        print number_servers
        node_name = sys.argv[2]
        print node_name
        node_prefix = ""
        if(len(sys.argv) == 4):
           node_prefix = sys.argv[3]
           print node_prefix
        else:
           print "Node prefix was not defined"
      
    main (cluster_name, number_servers, node_prefix, node_name)


